<?php
require("wordnik.api.php");

$q=$argv[1]; //the command from Alfred

//grab key from query
//you can hard-code your key if you want
$k=$argv[2];

global $game;
$game = json_decode(file_get_contents('hangman.txt'));
$stats = json_decode(file_get_contents('stats.txt'));
if (!isset($game->word)) {
	$q = 'new';
}
if (!isset($stats->wins)) {
	$stats = reset_stats();
}

switch ($q) {
	case 'new':
		$game = new_game( $k );
		$message = 'New game started.';
		break;
	case 'board':
		$message = isset($game->message) ? $game->message : "Current Game:";
		break;
	case 'stats':
		$message = "Wins: $stats->wins\nLosses: $stats->losses\n";
		break;
	case 'resetstats':
		$stats = reset_stats();
		$message = "Wins: $stats->wins\nLosses: $stats->losses\n";
		break;
	case 'answer':
		if (($game->turns < 1) || $game->solved) {
			$message = "The word was: $game->word\n";
			$message .= "Definition: $game->definition\n";
		} else {
			$message = "You haven't run out of turns yet!\n";
			//$game->solved = true;
		}
		break;
	case 'magic!':
		echo "$game->word\n";
		echo "Definition: $game->definition";
		die();
		break;
	case 'help':
		echo "new: start new game\n";
		echo "board: show current gameboard\n";
		echo "stats: show current win/loss stats\n";
		echo "resetstats: reset stats\n";
		echo "answer: show answer (after losing)\n";
		echo "{single letter}: your guess\n";
		echo "help: this";
		die();
		break;
	default:
		$q = strtolower($q);
		if ($game->turns < 1) {
			$message = 'You have run out of turns.';
			break;
		}
		if (!in_array($q, range('a','z'))) {
			$message = 'Not a valid play.';
			break;
		}
		if (in_array($q, $game->guesses)) {
			$message = 'You have already played that.';
			break;
		}

		$game->guesses[] = $q;
		$found = add_letter( $q );
		if ($found) {
			$message = 'You played "'.$q.'" successfully.';
		}
		else {
			$message = 'Sorry, no "'.$q.'".';
			--$game->turns;
		}
		//if no more underscores, we've solved it!
		if (array_search('_', $game->known) === false) {
			++$stats->wins;
			$message = "Solved!\n". implode($game->known);
			$game->message = $message;
			$game->solved = true;
		}
		else if ($game->turns < 1) {
			++$stats->losses;
			$message = "Sorry! You lose.\n". implode($game->known);
		}

		break;
}

//append stats to each result
$message .="\nSo far: ". implode(' ',$game->known)." $game->count
$game->turns turns remaining.
Guesses: ". implode(', ',$game->guesses)."
";

//update game data, stats
fwrite(fopen('hangman.txt','w'), json_encode($game));
fwrite(fopen('stats.txt','w'), json_encode($stats));

//send message
die( $message );

function add_letter($letter, $found = false) {
	global $game;
	//there's probably an easier way to do this...
	$pos = strpos($game->hold, $letter);
	if ($pos !== false) {
		$found = true;
		$split = str_split($game->hold, $pos+1);
		$split[0] = substr($split[0], 0, -1).'_';
		$game->hold = implode($split);
		$game->known[$pos] = $letter;

		add_letter($letter, $found);
	}

	return $found;
}

function new_game( $api_key ) {

	$wordnik = Wordnik::instance($api_key);
	$random = $wordnik->getRandomWord();
	$definition = $wordnik->getDefinitions( $random->wordstring );
	
	$game->word = $random->wordstring;
	$game->hold = $game->word;
	$game->known = array_fill(0, strlen($game->word), '_');
	$game->count = ' ('.strlen($game->word).' letters)';
	$game->turns = 6;
	$game->guesses = array();
	$game->definition = $definition[0]->text;
	$game->solved = false;
	
	$pattern ='/^([a-zA-Z]*)$/';
	preg_match($pattern, $game->word, $matches);
	//in case we've grabbed a word with punctuation or other special characters
	if (count($matches) < 1)
		$game = new_game( $api_key );

	return $game;
}

function reset_stats() {

	$stats->wins = 0;
	$stats->losses = 0;

	return $stats;
}